# Generated by Django 2.2.12 on 2021-06-16 08:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('moneta', '0003_auto_20210530_1400'),
    ]

    operations = [
        migrations.AlterField(
            model_name='invoice',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='status',
            field=models.CharField(choices=[('CHECK', 'CHECK'), ('PAID', 'PAID')], default='CHECK', max_length=6),
        ),
        migrations.AlterField(
            model_name='paymentsystem',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]
